package graphics.grapher;

import gui.ClosableJFrame;

import java.awt.*;

/**
 * This class contains the graphcomponent, which has the
 * axix, lbels and images of data to be graphed.
 */
public class GraphFrame extends ClosableJFrame {

    private GraphComponent graphComponent;

    private Axis xaxis;
    private Axis yaxis;
    private int np;
    private ClosableJFrame gf = new ClosableJFrame("Graph Frame");
    private DataSet dataSet;
    private double data[];

    public GraphFrame(DoubleXYData doubleXYData) {
        super("Graph Frame");
        np = doubleXYData.getNp();
        doubleXYData.checkConsistent();
        //dl.print();
        initialize();

        data = new double[np * 2];
        makeAxis(doubleXYData);
    }

    void makeAxis(DoubleXYData doubleXYData) {
        int ix = 0;
        int iy = 1;
        for (int i = 0; i < np; i++) {
            data[ix] = doubleXYData.getX()[i];
            data[iy] = doubleXYData.getY()[i];
            ix = ix + 2;
            iy = iy + 2;
        }
        graphComponent.setBackground(Color.white);
        dataSet = graphComponent.loadDataSet(data, np);
        dataSet.linecolor = new Color(0, 0, 0);
        dataSet.legendColor(new Color(0, 0, 0));




        // xaxis_t = graphComponent.createAxis(Axis.TOP);
        // xaxis_t.attachDataSet(data1);
        // xaxis_t.title = dl.x_label;

        xaxis = graphComponent.createAxis(Axis.BOTTOM);
        xaxis.attachDataSet(dataSet);
        xaxis.title = doubleXYData.getXLabel();

        yaxis = graphComponent.createAxis(Axis.RIGHT);
        yaxis.attachDataSet(dataSet);
        yaxis.title = doubleXYData.getYLabel();

    }
    public void setImageData(Image img){
        graphComponent.setImageData(img);
    }
    private void initialize() {
        graphComponent = new GraphComponent();
        // gf is graphics frame canvas will be drawn on
        // lets make a small graph.
        //gf.add("Center", graphComponent);
        gf.addComponent("Center", graphComponent);
        gf.setSize(250, 250);
        gf.show();
    }

    public void draw() {
        gf.repaint();
    }

}